from urllib.parse import urljoin

from django.utils.translation import gettext as _

from api_fhir_r4.configurations import GeneralConfiguration, R4InvoiceConfig


class InvoiceChargeItemMapping(object):
    SYSTEM = urljoin(
        GeneralConfiguration.get_system_base_url(),
        R4InvoiceConfig.get_fhir_invoice_charge_item_system(),
    )
    charge_item = {
        "policy": {"code": "policy", "display": _("Policy"), "system": SYSTEM},
        "contractcontributionplandetails": {
            "code": "contribution",
            "display": _("Contribution"),
            "system": SYSTEM,
        },
    }


class InvoiceTypeMapping(object):
    SYSTEM = urljoin(
        GeneralConfiguration.get_system_base_url(),
        R4InvoiceConfig.get_fhir_invoice_type_system(),
    )

    invoice_type = {
        "family": {
            "code": "contribution",
            "display": _("Contribution"),
            "system": SYSTEM,
        },
        "contract": {"code": "contract", "display": _("Contract"), "system": SYSTEM},
    }


class BillChargeItemMapping(object):
    SYSTEM = urljoin(
        GeneralConfiguration.get_system_base_url(),
        R4InvoiceConfig.get_fhir_bill_charge_item_system(),
    )
    charge_item = {
        "claim": {"code": "claim", "display": _("Claim"), "system": SYSTEM},
        "commission": {
            "code": "commission",
            "display": _("Commission"),
            "system": SYSTEM,
        },
        "capitationpayment": {"code": "claim", "display": _("Claim"), "system": SYSTEM},
        "policy": {"code": "policy", "display": _("Policy"), "system": SYSTEM},
    }


class BillTypeMapping(object):
    SYSTEM = urljoin(
        GeneralConfiguration.get_system_base_url(),
        R4InvoiceConfig.get_fhir_bill_type_system(),
    )

    invoice_type = {
        "batchrun": {
            "code": "claim-batch",
            "display": _("Claim batch payment"),
            "system": SYSTEM,
        },
        "commission": {
            "code": "commission",
            "display": _("Commission"),
            "system": SYSTEM,
        },
        "fees": {"code": "fees", "display": _("Fees"), "system": SYSTEM},
        "policy": {"code": "policy", "display": _("Policy"), "system": SYSTEM},
    }



from api_fhir_r4.configurations import OrganisationConfiguration


class R4OrganisationConfig(OrganisationConfiguration):

    @classmethod
    def build_configuration(cls, cfg):
        cls.get_config().R4_fhir_organisation_config = cfg[
            "R4_fhir_organisation_config"
        ]

    @classmethod
    def get_fhir_ph_organisation_type(cls):
        return cls.get_config_attribute("R4_fhir_organisation_config").get(
            "fhir_ph_organisation_type", "bus"
        )

    @classmethod
    def get_fhir_ph_organisation_type_system(cls):
        return cls.get_config_attribute("R4_fhir_organisation_config").get(
            "fhir_ph_organisation_type_system",
            "http://terminology.hl7.org/CodeSystem/organization-type",
        )

    @classmethod
    def get_fhir_ph_organisation_contactentity_type(cls):
        return cls.get_config_attribute("R4_fhir_organisation_config").get(
            "fhir_ph_organisation_type_contactentity_type",
            "http://terminology.hl7.org/CodeSystem/contactentity-type",
        )

    @classmethod
    def get_fhir_address_municipality_extension_system(cls):
        return cls.get_config_attribute("R4_fhir_organisation_config").get(
            "fhir_address_municipality_extension_system",
            "StructureDefinition/address-municipality",
        )

    @classmethod
    def get_fhir_location_reference_extension_system(cls):
        return cls.get_config_attribute("R4_fhir_organisation_config").get(
            "fhir_location_reference_extension_system",
            "StructureDefinition/address-location-reference",
        )

    @classmethod
    def get_fhir_ph_organisation_legal_form_extension_system(cls):
        return cls.get_config_attribute("R4_fhir_organisation_config").get(
            "fhir_ph_organisation_legal_form_extension_system",
            "StructureDefinition/organization-ph-legal-form",
        )

    @classmethod
    def get_fhir_ph_organisation_activity_extension_system(cls):
        return cls.get_config_attribute("R4_fhir_organisation_config").get(
            "fhir_ph_organisation_activity_extension_system",
            "StructureDefinition/organization-ph-activity",
        )

    @classmethod
    def get_fhir_ph_organisation_legal_form_code_system(cls):
        return cls.get_config_attribute("R4_fhir_organisation_config").get(
            "fhir_ph_organisation_legal_form_code_system",
            "CodeSystem/organization-ph-legal-form",
        )

    @classmethod
    def get_fhir_ph_organisation_activity_code_system(cls):
        return cls.get_config_attribute("R4_fhir_organisation_config").get(
            "fhir_ph_organisation_activity_code_system",
            "CodeSystem/organization-ph-activity",
        )

    @classmethod
    def get_fhir_insurer_organisation_id(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "id", "openIMIS-Implementation"
        )

    @classmethod
    def get_fhir_insurer_organisation_code(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "code", "openIMIS"
        )

    @classmethod
    def get_fhir_insurer_organisation_name(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "name", "openIMIS Implementation"
        )

    @classmethod
    def get_fhir_insurer_organisation_type(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "type", "ins"
        )

    @classmethod
    def get_fhir_insurer_organisation_email(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "email", "implementation@openimis.org"
        )

    @classmethod
    def get_fhir_insurer_organisation_phone(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "phone", "0908060703"
        )

    @classmethod
    def get_fhir_insurer_organisation_fax(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "fax", "0908060730"
        )

    @classmethod
    def get_fhir_insurer_organisation_contact_name(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "contact_name", "Manuel D. Medina"
        )

    @classmethod
    def get_fhir_insurer_organisation_municipality(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "municipality", "Jamu"
        )

    @classmethod
    def get_fhir_insurer_organisation_city(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "city", "Jamula"
        )

    @classmethod
    def get_fhir_insurer_organisation_district(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "district", "Rapta"
        )

    @classmethod
    def get_fhir_insurer_organisation_state(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "state", "Ultha"
        )

    @classmethod
    def get_fhir_insurer_organisation_line(cls):
        return cls.get_config_attribute("R4_fhir_insurance_organisation_config").get(
            "line", "1 Pasay"
        )



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), "README.md")) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name="openimis-be-api_fhir_r4",
    version="1.10.0",
    packages=find_packages(),
    include_package_data=True,
    license="GNU AGPL v3",
    description="The openIMIS Backend FHIR R4 API reference module.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://openimis.org/",
    author="Faris Ahmetasevic",
    author_email="faris.ahmetasevic@hotmail.com",
    install_requires=[
        "django",
        "django-db-signals",
        "djangorestframework",
        "pydantic<2",
        "aiohttp~=3.13.3",
        "fhir.resources~=7.0.2",
        "asynctest==0.13.0",
        "openimis-be-core",
        "openimis-be-insuree",
        "openimis-be-location",
        "openimis-be-claim",
        "openimis-be-policy",
        "openimis-be-policyholder",
        "orjson",
    ],
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.1",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)



from typing import Dict, Type
from api_fhir_r4.containedResources.containedResources import (
    AbstractContainedResourceCollection,
    ContainedResourceDefinition,
)

from api_fhir_r4.serializers.baseSerializer import BaseFHIRSerializer
from api_fhir_r4.serializers.patientSerializer import PatientSerializer
from api_fhir_r4.serializers.groupSerializer import GroupSerializer
from api_fhir_r4.serializers.healthFacilityOrganisationSerializer import HealthFacilityOrganisationSerializer
from api_fhir_r4.serializers.claimAdminPractitionerSerializer import ClaimAdminPractitionerSerializer
from api_fhir_r4.serializers.medicationSerializer import MedicationSerializer
from api_fhir_r4.serializers.activityDefinitionSerializer import ActivityDefinitionSerializer
from api_fhir_r4.serializers.claimAdminPractitionerRoleSerializer import ClaimAdminPractitionerRoleSerializer
from medical.models import Item, Service


class ClaimContainedResources(AbstractContainedResourceCollection):
    @classmethod
    def _definitions_for_serializers(
        cls,
    ) -> Dict[Type[BaseFHIRSerializer], ContainedResourceDefinition]:
        return {
            PatientSerializer: ContainedResourceDefinition("insuree", "Patient"),
            GroupSerializer: ContainedResourceDefinition(
                "insuree",
                "Group",
                lambda model, field: model.__getattribute__(field).family,
            ),
            HealthFacilityOrganisationSerializer: ContainedResourceDefinition(
                "health_facility", "Organization"
            ),
            ClaimAdminPractitionerSerializer: ContainedResourceDefinition(
                "admin", "Practitioner"
            ),
            ClaimAdminPractitionerRoleSerializer: ContainedResourceDefinition(
                "admin", "PractitionerRole"
            ),
            MedicationSerializer: ContainedResourceDefinition(
                "items",
                "Medication",
                lambda model, field: [
                    item.item
                    for item in model.__getattribute__(field).filter(*Item.filter_validity())
                ],
            ),
            ActivityDefinitionSerializer: ContainedResourceDefinition(
                "services",
                "ActivityDefinition",
                lambda model, field: [
                    service.service
                    for service in model.__getattribute__(field).filter(
                        *Service.filter_validity()
                    )
                ],
            ),
        }



from rest_framework.test import APITestCase
from api_fhir_r4.configurations import GeneralConfiguration
from api_fhir_r4.tests import (
    GenericFhirAPITestMixin,
    FhirApiCreateTestMixin,
    FhirApiReadTestMixin,
    FhirApiDeleteTestMixin,
    FhirApiUpdateTestMixin,
)
from api_fhir_r4.tests.mixin.SubscriptionTestMixin import SubscriptionTestMixin
from api_fhir_r4.tests.mixin.logInMixin import LogInMixin


class SubscriptionAPITests(
    GenericFhirAPITestMixin,
    APITestCase,
    LogInMixin,
    FhirApiCreateTestMixin,
    FhirApiReadTestMixin,
    FhirApiDeleteTestMixin,
    FhirApiUpdateTestMixin,
    SubscriptionTestMixin,
):
    base_url = GeneralConfiguration.get_base_url() + "Subscription/"
    _test_json_path = "/test/test_subscription.json"
    _updated_endpoint = "https://modifiedendpoint.com"

    def update_resource(self, data):
        data["channel"]["endpoint"] = self._updated_endpoint

    def verify_updated_obj(self, obj):
        self.assertEqual(obj.channel.endpoint, self._updated_endpoint)

    def setUp(self):
        super(SubscriptionAPITests, self).setUp()
        user = self.get_or_create_user_api()
        subscription = self.create_test_imis_instance()
        subscription.id = None
        subscription.save(username=user.username)

    def get_id_for_created_resource(self, response):
        return response.data["id"]



from rest_framework import status
from rest_framework.test import APITestCase
from api_fhir_r4.tests import GenericFhirAPITestMixin
from api_fhir_r4.configurations import GeneralConfiguration
from insuree.models import Education
from insuree.test_helpers import create_test_education


class CodeSystemPatientEducationLevelAPITests(GenericFhirAPITestMixin, APITestCase):
    base_url = (
        GeneralConfiguration.get_base_url() + "CodeSystem/patient-education-level/"
    )

    def setUp(self):
        super(CodeSystemPatientEducationLevelAPITests, self).setUp()
        self._EXPECTED_COUNT = Education.objects.all().count()

    def test_get_bad_authorization(self):
        response = self.client.get(self.base_url, data=None, format="json")
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_get_should_return_code_system(self):
        self.login()
        response = self.client.get(self.base_url, data=None, format="json")
        response_data = response.json()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        create_test_education()
        nursery = ""
        for concept in response_data["concept"]:
            if concept["display"] == "Nursery":
                nursery = concept["display"]
        self.assertEqual(nursery, "Nursery")
        self.assertEqual(response_data["count"], self._EXPECTED_COUNT)
        self.assertEqual(response_data["name"], "PatientEducationLevelCS")
        self.assertEqual(response_data["title"], "Education Level (Patient)")



from rest_framework import status
from rest_framework.test import APITestCase
from api_fhir_r4.tests import GenericFhirAPITestMixin
from api_fhir_r4.configurations import GeneralConfiguration
from medical.models import Service
from medical.test_helpers import create_test_service


class CodeSystemMedicalServiceAPITests(GenericFhirAPITestMixin, APITestCase):
    base_url = (
        GeneralConfiguration.get_base_url() + "CodeSystem/medical-service/"
    )

    def setUp(self):
        create_test_service("S")
        super(CodeSystemMedicalServiceAPITests, self).setUp()
        self._EXPECTED_COUNT = Service.objects.all().count()

    def test_get_bad_authorization(self):
        response = self.client.get(self.base_url, data=None, format="json")
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_get_should_return_code_system(self):
        self.login()
        response = self.client.get(self.base_url, data=None, format="json")
        response_data = response.json()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        # service_code = ""
        # for concept in response_data["concept"]:
        #     if concept["display"].startswith("test S service"):
        #         service_code = concept["code"]
#        self.assertTrue(service_code.startswith("TS-"))
        self.assertEqual(response_data["count"], self._EXPECTED_COUNT)
        self.assertEqual(response_data["name"], "MedicalServiceCS")
        self.assertEqual(response_data["title"], "Medical Service")



# Generated by Django 4.2.27 on 2026-01-13 20:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("api_fhir_r4", "0007_alter_historicalsubscription_criteria_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubscription",
            name="channel",
            field=models.SmallIntegerField(
                choices=[(1, "rest-hook")], db_column="Channel"
            ),
        ),
        migrations.AlterField(
            model_name="subscription",
            name="channel",
            field=models.SmallIntegerField(
                choices=[(1, "rest-hook")], db_column="Channel"
            ),
        ),
    ]



from core.models.user import ClaimAdmin
from django.utils.translation import gettext

from api_fhir_r4.converters import ClaimAdminPractitionerRoleConverter
from api_fhir_r4.exceptions import FHIRRequestProcessException
from api_fhir_r4.serializers.baseSerializer import BaseFHIRSerializer


class ClaimAdminPractitionerRoleSerializer(BaseFHIRSerializer):

    fhirConverter = ClaimAdminPractitionerRoleConverter

    def create(self, validated_data):
        claim_admin_id = validated_data.get("id")
        claim_admin = None
        if claim_admin_id:
            claim_admin = ClaimAdmin.objects.get(pk=claim_admin_id)
        if not claim_admin:
            raise FHIRRequestProcessException(
                [gettext("Missing Practitioner for id {}").format(claim_admin_id)]
            )
        claim_admin.health_facility_id = validated_data.get(
            "health_facility_id", claim_admin.health_facility_id
        )
        claim_admin.save()
        return claim_admin

    def update(self, instance, validated_data):
        instance.health_facility_id = validated_data.get(
            "health_facility_id", instance.health_facility_id
        )
        instance.audit_user_id = self.get_audit_user_id()
        instance.save()
        return instance



import datetime

from django.db.models import Prefetch
from rest_framework.response import Response
from rest_framework.serializers import ValidationError
from rest_framework.viewsets import GenericViewSet

from api_fhir_r4.mixins import MultiIdentifierRetrieverMixin, ListModelMixin
from api_fhir_r4.model_retrievers import (
    UUIDIdentifierModelRetriever,
    CodeIdentifierModelRetriever,
)
from api_fhir_r4.permissions import FHIRApiClaimPermissions
from api_fhir_r4.serializers import ClaimSerializer
from api_fhir_r4.views.fhir.base import BaseFHIRView
from api_fhir_r4.views.filters import ValidityFromRequestParameterFilter
from claim.models import Claim, ClaimItem, ClaimService
from insuree.models import Insuree, InsureePolicy


class ClaimViewSet(
    BaseFHIRView, MultiIdentifierRetrieverMixin, ListModelMixin, GenericViewSet
):
    retrievers = [UUIDIdentifierModelRetriever, CodeIdentifierModelRetriever]
    serializer_class = ClaimSerializer
    permission_classes = (FHIRApiClaimPermissions,)

    def list(self, request, *args, **kwargs):
        queryset = self.get_queryset()
        refDate = request.GET.get("refDate")
        identifier = request.GET.get("identifier")
        patient = request.GET.get("patient")
        contained = bool(request.GET.get("contained"))

        if identifier is not None:
            return self.retrieve(request, *args, **{**kwargs, "identifier": identifier})
        else:
            queryset = queryset.filter(validity_to__isnull=True).order_by(
                "validity_from"
            )
            if refDate is not None:
                try:
                    date_from = datetime.datetime.strptime(refDate, "%Y-%m-%d").date()
                    queryset = queryset.filter(validity_from__gte=date_from)
                except ValueError:
                    raise ValidationError(
                        {
                            "refDate": 'Invalid date format, should be in "%Y-%m-%d" format'
                        }
                    )

            if patient is not None:
                for_patient = Insuree.objects.get(uuid=patient)
                queryset = queryset.filter(insuree=for_patient)

        serializer = ClaimSerializer(
            self.paginate_queryset(queryset),
            many=True,
            context={"contained": contained},
            user=request.user,
        )
        return self.get_paginated_response(serializer.data)

    def retrieve(self, request, *args, **kwargs):
        contained = bool(request.GET.get("contained"))
        ref_type, instance = self._get_object_with_first_valid_retriever(
            kwargs["identifier"]
        )
        serializer = self.get_serializer(
            instance,
            context={"contained": contained},
            reference_type=ref_type,
            user=request.user,
        )
        return Response(serializer.data)

    def get_queryset(self):
        queryset = (
            Claim.get_queryset(None, self.request.user)
            .order_by("validity_from")
            .select_related("insuree")
            .select_related("health_facility")
            .select_related("icd")
            .select_related("icd_1")
            .select_related("icd_2")
            .select_related("icd_3")
            .select_related("icd_4")
            .prefetch_related(
                Prefetch(
                    "items", queryset=ClaimItem.objects.filter(validity_to__isnull=True)
                )
            )
            .prefetch_related(
                Prefetch(
                    "services",
                    queryset=ClaimService.objects.filter(validity_to__isnull=True),
                )
            )
            .prefetch_related(
                Prefetch(
                    "insuree__insuree_policies",
                    queryset=InsureePolicy.objects.filter(
                        validity_to__isnull=True
                    ).select_related("policy"),
                )
            )
        )
        return ValidityFromRequestParameterFilter(self.request).filter_queryset(
            queryset
        )



from api_fhir_r4.converters import ClaimAdminPractitionerRoleConverter
from fhir.resources.R4B.practitionerrole import PractitionerRole
from api_fhir_r4.tests import ClaimAdminPractitionerRoleTestMixin
from api_fhir_r4.tests.mixin import (
    ConvertJsonToFhirTestMixin,
    ConvertToFhirTestMixin,
    ConvertToImisTestMixin,
)


class ClaimAdminPractitionerRoleConverterTestCase(
    ClaimAdminPractitionerRoleTestMixin,
    ConvertToImisTestMixin,
    ConvertToFhirTestMixin,
    ConvertJsonToFhirTestMixin,
):
    converter = ClaimAdminPractitionerRoleConverter
    fhir_resource = PractitionerRole
    json_repr = "test/test_claimAdminPractitionerRole.json"



from rest_framework import status
from rest_framework.test import APITestCase
from api_fhir_r4.tests import GenericFhirAPITestMixin
from api_fhir_r4.configurations import GeneralConfiguration
from insuree.models import ConfirmationType


class CodeSystemGroupConfirmationTypeAPITests(GenericFhirAPITestMixin, APITestCase):
    base_url = (
        GeneralConfiguration.get_base_url() + "CodeSystem/group-confirmation-type/"
    )

    def setUp(self):
        super(CodeSystemGroupConfirmationTypeAPITests, self).setUp()
        self._EXPECTED_COUNT = ConfirmationType.objects.all().count()

    def test_get_bad_authorization(self):
        response = self.client.get(self.base_url, data=None, format="json")
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_get_should_return_code_system(self):
        self.login()
        response = self.client.get(self.base_url, data=None, format="json")
        response_data = response.json()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response_data["count"], self._EXPECTED_COUNT)
        self.assertEqual(response_data["name"], "GroupConfirmationTypeCS")
        self.assertEqual(response_data["title"], "Confirmation Types (Group)")



from urllib.parse import urljoin

from django.utils.translation import gettext as _
from fhir.resources.R4B.address import Address
from fhir.resources.R4B.extension import Extension
from fhir.resources.R4B.humanname import HumanName

from api_fhir_r4.mapping.organizationMapping import (
    PolicyHolderOrganisationLegalFormMapping,
    PolicyHolderOrganisationActivityMapping,
)
from api_fhir_r4.models.imisModelEnums import (
    ImisLocationType,
    ContactPointSystem,
    AddressType,
)
from location.models import Location
from policyholder.models import PolicyHolder
from api_fhir_r4.configurations import (
    R4IdentifierConfig,
    R4OrganisationConfig,
    GeneralConfiguration,
)
from api_fhir_r4.converters import BaseFHIRConverter, ReferenceConverterMixin
from fhir.resources.R4B.organization import Organization
from api_fhir_r4.utils import DbManagerUtils


class PolicyHolderOrganisationConverter(BaseFHIRConverter, ReferenceConverterMixin):
    @classmethod
    def to_fhir_obj(
        cls,
        imis_organisation,
        reference_type=ReferenceConverterMixin.UUID_REFERENCE_TYPE,
    ):
        fhir_organisation = Organization()
        cls.build_fhir_pk(fhir_organisation, imis_organisation, reference_type)
        cls.build_fhir_extensions(fhir_organisation, imis_organisation)
        cls.build_fhir_identifiers(fhir_organisation, imis_organisation, reference_type)
        cls.build_fhir_type(fhir_organisation)
        cls.build_fhir_name(fhir_organisation, imis_organisation)
        cls.build_fhir_telecom(fhir_organisation, imis_organisation)
        cls.build_fhir_ph_address(fhir_organisation, imis_organisation, reference_type)
        cls.build_fhir_contact(fhir_organisation, imis_organisation)
        return fhir_organisation

    @classmethod
    def to_imis_obj(cls, fhir_organisation, audit_user_id):
        errors = []
        fhir_org = Organization(**fhir_organisation)
        imis_ph = PolicyHolder()
        imis_ph.audit_user_id = audit_user_id

        cls.build_imis_ph_identifier(imis_ph, fhir_org, errors)
        cls.build_imis_ph_name(imis_ph, fhir_org, errors)
        cls.build_imis_ph_legal_form(imis_ph, fhir_org, errors)
        cls.build_imis_ph_activity(imis_ph, fhir_org, errors)
        cls.build_imis_ph_telecom(imis_ph, fhir_org, errors)
        cls.build_imis_ph_address(imis_ph, fhir_org, errors)
        cls.build_imis_ph_contact(imis_ph, fhir_org, errors)

        cls.check_errors(errors)
        return imis_ph

    @classmethod
    def get_imis_obj_by_fhir_reference(cls, reference, errors=None):
        return DbManagerUtils.get_object_or_none(
            PolicyHolder,
            **cls.get_database_query_id_parameteres_from_reference(reference)
        )

    @classmethod
    def get_reference_obj_id(cls, obj):
        return obj.uuid

    @classmethod
    def get_reference_obj_uuid(cls, obj):
        return obj.uuid

    @classmethod
    def get_reference_obj_code(cls, obj):
        return obj.code

    @classmethod
    def get_fhir_code_identifier_type(cls):
        return R4IdentifierConfig.get_fhir_generic_type_code()

    @classmethod
    def get_fhir_resource_type(cls):
        return Organization

    @classmethod
    def build_fhir_identifiers(
        cls, fhir_organisation, imis_organisation, reference_type
    ):
        identifiers = []
        cls.build_all_identifiers(identifiers, imis_organisation, reference_type)
        fhir_organisation.identifier = identifiers

    @classmethod
    def build_fhir_extensions(cls, fhir_organisation, imis_organisation):
        if imis_organisation.legal_form:
            cls.build_fhir_legal_form_extension(fhir_organisation, imis_organisation)
        if imis_organisation.activity_code:
            cls.build_fhir_activity_extension(fhir_organisation, imis_organisation)

    @classmethod
    def build_fhir_legal_form_extension(cls, fhir_organisation, imis_organisation):
        codeable_concept = cls.build_codeable_concept_from_coding(
            cls.build_fhir_mapped_coding(
                PolicyHolderOrganisationLegalFormMapping.fhir_ph_code_system(
                    imis_organisation.legal_form
                )
            )
        )
        base = GeneralConfiguration.get_system_base_url()
        url = urljoin(
            base,
            R4OrganisationConfig.get_fhir_ph_organisation_legal_form_extension_system(),
        )
        extension = cls.build_fhir_codeable_concept_extension(codeable_concept, url)
        if isinstance(fhir_organisation.extension, list):
            fhir_organisation.extension.append(extension)
        else:
            fhir_organisation.extension = [extension]

    @classmethod
    def build_fhir_activity_extension(cls, fhir_organisation, imis_organisation):
        codeable_concept = cls.build_codeable_concept_from_coding(
            cls.build_fhir_mapped_coding(
                PolicyHolderOrganisationActivityMapping.fhir_ph_code_system(
                    imis_organisation.activity_code
                )
            )
        )
        base = GeneralConfiguration.get_system_base_url()
        url = urljoin(
            base,
            R4OrganisationConfig.get_fhir_ph_organisation_activity_extension_system(),
        )
        extension = cls.build_fhir_codeable_concept_extension(codeable_concept, url)
        if isinstance(fhir_organisation.extension, list):
            fhir_organisation.extension.append(extension)
        else:
            fhir_organisation.extension = [extension]

    @classmethod
    def build_fhir_type(cls, fhir_organisation):
        fhir_organisation.type = [
            cls.build_codeable_concept(
                R4OrganisationConfig.get_fhir_ph_organisation_type(),
                system=R4OrganisationConfig.get_fhir_ph_organisation_type_system(),
            )
        ]

    @classmethod
    def build_fhir_name(cls, fhir_organisation, imis_organisation):
        fhir_organisation.name = imis_organisation.trade_name

    @classmethod
    def build_fhir_telecom(cls, fhir_organisation, imis_organisation):
        fhir_organisation.telecom = []
        if imis_organisation.email:
            fhir_organisation.telecom.append(
                cls.build_fhir_contact_point(
                    system=ContactPointSystem.EMAIL, value=imis_organisation.email
                )
            )
        if imis_organisation.fax:
            fhir_organisation.telecom.append(
                cls.build_fhir_contact_point(
                    system=ContactPointSystem.FAX, value=imis_organisation.fax
                )
            )
        if imis_organisation.phone:
            fhir_organisation.telecom.append(
                cls.build_fhir_contact_point(
                    system=ContactPointSystem.PHONE, value=imis_organisation.phone
                )
            )

    @classmethod
    def build_fhir_ph_address(
        cls, fhir_organisation, imis_organisation, reference_type
    ):
        address = Address.construct()
        address.type = AddressType.PHYSICAL.value
        if imis_organisation.address and "address" in imis_organisation.address:
            address.line = [imis_organisation.address["address"]]
        fhir_organisation.address = [address]
        if imis_organisation.locations:
            cls.build_fhir_address_field(fhir_organisation, imis_organisation.locations)
            cls.build_fhir_location_extension(
                fhir_organisation, imis_organisation, reference_type
            )

    @classmethod
    def build_fhir_address_field(cls, fhir_organisation, location: Location):
        current_location = location
        while current_location:
            if current_location.type == ImisLocationType.REGION.value:
                fhir_organisation.address[0].state = current_location.name
            elif current_location.type == ImisLocationType.DISTRICT.value:
                fhir_organisation.address[0].district = current_location.name
            elif current_location.type == ImisLocationType.WARD.value:
                cls.build_fhir_municipality_extension(
                    fhir_organisation, current_location
                )
            elif current_location.type == ImisLocationType.VILLAGE.value:
                fhir_organisation.address[0].city = current_location.name
            current_location = current_location.parent

    @classmethod
    def build_fhir_municipality_extension(
        cls, fhir_organisation, municipality: Location
    ):
        extension = Extension.construct()
        base = GeneralConfiguration.get_system_base_url()
        extension.url = urljoin(
            base, R4OrganisationConfig.get_fhir_address_municipality_extension_system()
        )
        extension.valueString = municipality.name
        if isinstance(fhir_organisation.address[0].extension, list):
            fhir_organisation.address[0].extension.append(extension)
        else:
            fhir_organisation.address[0].extension = [extension]

    @classmethod
    def build_fhir_location_extension(
        cls, fhir_organisation, imis_organisation, reference_type
    ):
        base = GeneralConfiguration.get_system_base_url()
        url = urljoin(
            base, R4OrganisationConfig.get_fhir_location_reference_extension_system()
        )
        extension = cls.build_fhir_reference_extension(
            cls.build_fhir_resource_reference(
                imis_organisation.locations,
                type="Location",
                display=imis_organisation.locations.name,
                reference_type=reference_type,
            ),
            url,
        )
        if isinstance(fhir_organisation.address[0].extension, list):
            fhir_organisation.address[0].extension.append(extension)
        else:
            fhir_organisation.address[0].extension = [extension]

    @classmethod
    def build_fhir_contact(cls, fhir_organisation, imis_organisation):
        fhir_organisation.contact = []
        if imis_organisation.contact_name:
            name = HumanName.construct()
            name.text = "%s %s" % (
                imis_organisation.contact_name["name"],
                imis_organisation.contact_name["surname"],
            )
            fhir_organisation.contact.append({"name": name})

    @classmethod
    def build_imis_ph_identifier(cls, imis_ph, fhir_org, errors):
        value = cls.get_fhir_identifier_by_code(
            fhir_org.identifier, R4IdentifierConfig.get_fhir_generic_type_code()
        )
        if value:
            imis_ph.code = value
        cls.valid_condition(
            imis_ph.code is None, _("Missing PolicyHolder code"), errors
        )

    @classmethod
    def build_imis_ph_name(cls, imis_ph, fhir_org, errors):
        imis_ph.trade_name = fhir_org.name
        cls.valid_condition(
            imis_ph.trade_name is None, _("Missing PolicyHolder name"), errors
        )

    @classmethod
    def build_imis_ph_legal_form(cls, imis_ph, fhir_org, errors):
        if fhir_org.extension:
            ext_url_suffix = "organization-legal-form"
            legal_form_ext = next(
                (x for x in fhir_org.extension if ext_url_suffix in x.url), None
            )
            if legal_form_ext and hasattr(legal_form_ext, "valueCodeableConcept"):
                coding = cls.get_first_coding_from_codeable_concept(
                    legal_form_ext.valueCodeableConcept
                )
                if coding and coding.code:
                    imis_ph.legal_form = int(coding.code)

    @classmethod
    def build_imis_ph_activity(cls, imis_ph, fhir_org, errors):
        if fhir_org.extension:
            ext_url_suffix = "organization-activity"
            activity_ext = next(
                (x for x in fhir_org.extension if ext_url_suffix in x.url), None
            )
            if activity_ext and hasattr(activity_ext, "valueCodeableConcept"):
                coding = cls.get_first_coding_from_codeable_concept(
                    activity_ext.valueCodeableConcept
                )
                if coding and coding.code:
                    imis_ph.activity_code = int(coding.code)

    @classmethod
    def build_imis_ph_telecom(cls, imis_ph, fhir_org, errors):
        if fhir_org.telecom:
            for telecom in fhir_org.telecom:
                if telecom.system == ContactPointSystem.EMAIL.value:
                    imis_ph.email = telecom.value
                elif telecom.system == ContactPointSystem.PHONE.value:
                    imis_ph.phone = telecom.value
                elif telecom.system == ContactPointSystem.FAX.value:
                    imis_ph.fax = telecom.value

    @classmethod
    def build_imis_ph_address(cls, imis_ph, fhir_org, errors):
        if fhir_org.address and len(fhir_org.address) > 0:
            address = fhir_org.address[0]
            if address.line and len(address.line) > 0:
                imis_ph.address = {"address": address.line[0]}

    @classmethod
    def build_imis_ph_contact(cls, imis_ph, fhir_org, errors):
        if fhir_org.contact and len(fhir_org.contact) > 0:
            contact = fhir_org.contact[0]
            if hasattr(contact, "name") and contact.name:
                # Parse "FirstName LastName" format
                name_text = (
                    contact.name.text
                    if hasattr(contact.name, "text")
                    else str(contact.name)
                )
                name_parts = name_text.split(" ", 1)
                if len(name_parts) == 2:
                    imis_ph.contact_name = {
                        "name": name_parts[0],
                        "surname": name_parts[1],
                    }
                else:
                    imis_ph.contact_name = {"name": name_text, "surname": ""}




from api_fhir_r4.converters import InsurancePlanConverter

from fhir.resources.R4B.insuranceplan import InsurancePlan
from api_fhir_r4.tests import InsurancePlanTestMixin
from api_fhir_r4.tests.mixin import (
    ConvertToImisTestMixin,
    ConvertToFhirTestMixin,
    ConvertJsonToFhirTestMixin,
)


class InsurancePlanConverterTestCase(
    InsurancePlanTestMixin,
    ConvertToImisTestMixin,
    ConvertToFhirTestMixin,
    ConvertJsonToFhirTestMixin,
):
    converter = InsurancePlanConverter
    fhir_resource = InsurancePlan
    json_repr = "test/test_insurance_plan.json"



from copy import deepcopy
from urllib import parse
import datetime
from fhir.resources.R4B.subscription import Subscription as FHIRSubscription

from api_fhir_r4.configurations import R4SubscriptionConfig
from api_fhir_r4.converters import BaseFHIRConverter, ReferenceConverterMixin
from api_fhir_r4.exceptions import FHIRException
from api_fhir_r4.mapping.subscriptionMapping import (
    SubscriptionChannelMapping,
    SubscriptionStatusMapping,
)
from api_fhir_r4.models import Subscription
from api_fhir_r4.utils import TimeUtils


class SubscriptionConverter(BaseFHIRConverter):
    _error_unknown_imis_value = "Unknown imis `%(attr)s`: %(val)s"
    _error_invalid_attr = "Missing or invalid `%(attr)s` attribute"
    _error_forbidden_attr = "`%(attr)s` attribute forbidden"

    @classmethod
    def to_fhir_obj(
        cls,
        imis_subscription,
        reference_type=ReferenceConverterMixin.UUID_REFERENCE_TYPE,
    ):
        fhir_subscription = {}
        cls._build_fhir_id(fhir_subscription, imis_subscription)
        cls._build_fhir_status(fhir_subscription, imis_subscription)
        cls._build_fhir_end(fhir_subscription, imis_subscription)
        cls._build_fhir_reason(fhir_subscription, imis_subscription)
        cls._build_fhir_criteria(fhir_subscription, imis_subscription)
        cls._build_fhir_error(fhir_subscription, imis_subscription)
        cls._build_fhir_channel(fhir_subscription, imis_subscription)
        return FHIRSubscription.parse_obj(fhir_subscription)

    @classmethod
    def to_imis_obj(cls, fhir_subscription, audit_user_id):
        fhir_subscription = FHIRSubscription.parse_obj(fhir_subscription)
        imis_subscription = {}
        cls._build_imis_id(imis_subscription, fhir_subscription)
        cls._build_imis_status(imis_subscription, fhir_subscription)
        cls._build_imis_end(imis_subscription, fhir_subscription)
        cls._build_imis_criteria(imis_subscription, fhir_subscription)
        cls._validate_fhir_error(fhir_subscription)
        cls._build_imis_channel_type(imis_subscription, fhir_subscription)
        cls._build_imis_channel_endpoint(imis_subscription, fhir_subscription)
        cls._build_imis_channel_header(imis_subscription, fhir_subscription)
        return Subscription(**imis_subscription)

    @classmethod
    def get_fhir_code_identifier_type(cls):
        return FHIRException("Subscription resource does not contain code identifier")

    @classmethod
    def _build_fhir_id(cls, fhir_subscription, imis_subscription):
        fhir_subscription["id"] = str(imis_subscription.id)

    @classmethod
    def _build_fhir_status(cls, fhir_subscription, imis_subscription):
        if imis_subscription.status in SubscriptionStatusMapping.to_fhir_status:
            fhir_subscription["status"] = SubscriptionStatusMapping.to_fhir_status[
                imis_subscription.status
            ]
        else:
            raise FHIRException(
                cls._error_unknown_imis_value
                % {"attr": "status", "val": str(imis_subscription.status)}
            )

    @classmethod
    def _build_fhir_end(cls, fhir_subscription, imis_subscription):
        date = imis_subscription.expiring.astimezone().isoformat()
        fhir_subscription["end"] = date

    @classmethod
    def _build_fhir_reason(cls, fhir_subscription, imis_subscription):
        fhir_subscription["reason"] = imis_subscription.criteria[
            R4SubscriptionConfig.get_fhir_sub_criteria_key_resource()
        ]

    @classmethod
    def _build_fhir_criteria(cls, fhir_subscription, imis_subscription):
        criteria = deepcopy(imis_subscription.criteria)
        resource = criteria.pop(
            R4SubscriptionConfig.get_fhir_sub_criteria_key_resource()
        )
        resource_type = criteria.pop(
            R4SubscriptionConfig.get_fhir_sub_criteria_key_resource_type(), None
        )
        if resource_type:
            criteria["resourceType"] = resource_type
        fhir_subscription["criteria"] = parse.urlunparse(
            ("", "", resource, "", parse.urlencode(criteria, doseq=True), "")
        )

    @classmethod
    def _build_fhir_error(cls, fhir_subscription, imis_subscription):
        error = None  # TODO Add that after the error logging is added to the model
        if error:
            fhir_subscription["error"] = error

    @classmethod
    def _build_fhir_channel(cls, fhir_subscription, imis_subscription):
        fhir_channel = {}
        cls._build_fhir_channel_type(fhir_channel, imis_subscription)
        cls._build_fhir_channel_endpoint(fhir_channel, imis_subscription)
        cls._build_fhir_channel_header(fhir_channel, imis_subscription)
        fhir_subscription["channel"] = fhir_channel

    @classmethod
    def _build_fhir_channel_type(cls, fhir_channel, imis_subscription):
        if imis_subscription.channel in SubscriptionChannelMapping.to_fhir_channel:
            fhir_channel["type"] = SubscriptionChannelMapping.to_fhir_channel[
                imis_subscription.channel
            ]
        else:
            # Default to rest_hook for unknown channel values
            fhir_channel["type"] = (
                R4SubscriptionConfig.get_fhir_subscription_channel_rest_hook()
            )

    @classmethod
    def _build_fhir_channel_endpoint(cls, fhir_channel, imis_subscription):
        fhir_channel["endpoint"] = imis_subscription.endpoint

    @classmethod
    def _build_fhir_channel_header(cls, fhir_channel, imis_subscription):
        fhir_channel["header"] = [imis_subscription.headers]

    @classmethod
    def _build_imis_id(cls, imis_subscription, fhir_subscription):
        if fhir_subscription.id:
            imis_subscription["id"] = fhir_subscription.id

    @classmethod
    def _build_imis_status(cls, imis_subscription, fhir_subscription):
        if (
            fhir_subscription.status
            and fhir_subscription.status in SubscriptionStatusMapping.to_imis_status
        ):
            imis_subscription["status"] = SubscriptionStatusMapping.to_imis_status[
                fhir_subscription.status
            ]
        else:
            raise FHIRException(cls._error_invalid_attr % {"attr": "status"})

    @classmethod
    def _build_imis_end(cls, imis_subscription, fhir_subscription):
        if fhir_subscription.end:
            imis_subscription["expiring"] = TimeUtils.str_iso_to_date(
                fhir_subscription.end
            ).astimezone(datetime.timezone.utc)
        else:
            raise FHIRException(cls._error_invalid_attr % {"attr": "end"})

    @classmethod
    def _build_imis_criteria(cls, imis_subscription, fhir_subscription):
        if fhir_subscription.criteria:
            parsed_criteria = parse.urlparse(fhir_subscription.criteria)
            imis_criteria = dict(parse.parse_qsl(parsed_criteria.query))
            resource_type = imis_criteria.pop("resourceType", None)
            if resource_type:
                imis_criteria[
                    R4SubscriptionConfig.get_fhir_sub_criteria_key_resource_type()
                ] = resource_type
            imis_criteria[R4SubscriptionConfig.get_fhir_sub_criteria_key_resource()] = (
                parsed_criteria.path.strip("/")
            )
            imis_subscription["criteria"] = imis_criteria
            cls._validate_fhir_reason(imis_subscription, fhir_subscription)
        else:
            raise FHIRException(cls._error_invalid_attr % {"attr": "criteria"})

    @classmethod
    def _validate_fhir_reason(cls, imis_subscription, fhir_subscription):
        # there is no imis reason so validation only
        if "criteria" not in imis_subscription:
            raise FHIRException(cls._error_invalid_attr % {"attr": "criteria"})
        if (
            fhir_subscription.reason.lower()
            != imis_subscription["criteria"][
                R4SubscriptionConfig.get_fhir_sub_criteria_key_resource()
            ].lower()
        ):
            raise FHIRException(cls._error_invalid_attr % {"attr": "reason"})

    @classmethod
    def _validate_fhir_error(cls, fhir_subscription):
        # there should not be error attribute in fhir input
        if fhir_subscription.error:
            raise FHIRException(cls._error_forbidden_attr % {"attr": "error"})

    @classmethod
    def _build_imis_channel_type(cls, imis_subscription, fhir_subscription):
        if (
            hasattr(fhir_subscription.channel, "type")
            and fhir_subscription.channel.type
            and fhir_subscription.channel.type
            in SubscriptionChannelMapping.to_imis_channel
        ):
            imis_subscription["channel"] = SubscriptionChannelMapping.to_imis_channel[
                fhir_subscription.channel.type
            ]
        else:
            # Default to REST_HOOK if channel type is not specified or invalid
            imis_subscription["channel"] = (
                Subscription.SubscriptionChannel.REST_HOOK.value
            )

    @classmethod
    def _build_imis_channel_endpoint(cls, imis_subscription, fhir_subscription):
        if fhir_subscription.channel.endpoint:
            imis_subscription["endpoint"] = str(fhir_subscription.channel.endpoint)
        else:
            raise FHIRException(cls._error_invalid_attr % {"attr": "channel.endpoint"})

    @classmethod
    def _build_imis_channel_header(cls, imis_subscription, fhir_subscription):
        if fhir_subscription.channel.header:
            imis_subscription["headers"] = fhir_subscription.channel.header[0]



from typing import Union

from django.db.models import Q

from api_fhir_r4.configurations import R4SubscriptionConfig
from api_fhir_r4.models import Subscription
from core.datetimes.ad_datetime import datetime
from core.models import HistoryModel, VersionedModel


class SubscriptionCriteriaFilter:
    def __init__(
        self,
        imis_resource: Union[HistoryModel, VersionedModel],
        fhir_resource_name: str,
        fhir_resource_type_name: str,
    ):
        self.fhir_resource_name = fhir_resource_name
        self.fhir_resource_type_name = fhir_resource_type_name
        self.imis_resource = imis_resource

    def get_filtered_subscriptions(self):
        subscriptions = self._get_all_active_subscriptions()
        return self._get_matching_subscriptions(subscriptions)

    def _get_all_active_subscriptions(self):
        queryset = Subscription.objects.filter(
            status=Subscription.SubscriptionStatus.ACTIVE.value,
            expiring__gt=datetime.now(),
            is_deleted=False,
        )
        if self.fhir_resource_name:
            queryset = queryset.filter(
                criteria__contains={
                    R4SubscriptionConfig.get_fhir_sub_criteria_key_resource(): self.fhir_resource_name
                }
            )
        if self.fhir_resource_type_name:
            queryset = queryset.filter(
                ~Q(
                    criteria__jsoncontainskey=R4SubscriptionConfig.get_fhir_sub_criteria_key_resource_type()
                )
                | Q(
                    criteria__jsoncontains={
                        R4SubscriptionConfig.get_fhir_sub_criteria_key_resource_type(): self.fhir_resource_type_name
                    }
                )
            )
        return queryset.all()

    def _get_matching_subscriptions(self, subscriptions):
        return [
            subscription
            for subscription in subscriptions
            if self._is_matching_subscription(subscription)
        ]

    def _is_matching_subscription(self, sub):
        criteria = {
            criteria: sub.criteria[criteria]
            for criteria in sub.criteria
            if criteria != R4SubscriptionConfig.get_fhir_sub_criteria_key_resource()
            and criteria
            != R4SubscriptionConfig.get_fhir_sub_criteria_key_resource_type()
        }
        return not criteria or self._is_resource_matching_criteria(criteria)

    def _is_resource_matching_criteria(self, criteria):
        criteria["uuid"] = self.imis_resource.uuid
        return type(self.imis_resource).objects.filter(**criteria).exists()



import os
import json
from core.models import User
from api_fhir_r4.utils import DbManagerUtils
from core.services import create_or_update_interactive_user, create_or_update_core_user
from core.test_helpers import create_test_interactive_user, create_admin_role


def load_and_replace_json(path=None, sub_str={}):
    test_root = os.path.dirname(os.path.realpath(__file__))
    # removing the leading / of path
    path = path[1:] if path[0] == "/" else path
    json_representation = open(os.path.join(test_root, path)).read()

    for s_str in sub_str:
        json_representation = json_representation.replace(
            str(s_str), str(sub_str[s_str])
        )

    return json.loads(json_representation)


# example of use
#        self.sub_str[self._TEST_VILLAGE_UUID] = self.test_village.uuid
#        self.sub_str[self._TEST_SUBSTITUTION_OFFICER_UUID] = self.test_officer.uuid
#        self.sub_str[self._TEST_GROUP_UUID] = self.test_insuree.family.uuid
#        self.sub_str[self._TEST_INSUREE_UUID] = self.test_insuree.uuid
#        self.sub_str[self._TEST_INSUREE_CHFID] = self.test_insuree.chf_id
#        self.sub_str[self._TEST_OFFICER_UUID] = self.test_officer.uuid
#        self.sub_str[self._TEST_CLAIM_ADMIN_UUID] = self.test_claim_admin.uuid
#        self.sub_str[self._TEST_PRODUCT_UUID] = self.test_product.uuid
#
#        self.sub_str[self._TEST_HF_UUID] = self.test_hf.uuid
#
#        self.json_representation = load_and_replace_json(_test_json_request_path,self.sub_str)
_TEST_USER_NAME = "Admin"
_TEST_USER_PASSWORD = "admin123"
_TEST_DATA_USER = {
    "username": _TEST_USER_NAME,
    "password": _TEST_USER_PASSWORD,
    "language": "en",
    "roles": [create_admin_role().id],
}


def get_connection_payload(userdata=_TEST_DATA_USER):
    return {"username": userdata["username"], "password": userdata["password"]}


def get_or_create_user_api(userdata=_TEST_DATA_USER):
    user = create_test_interactive_user(**userdata)
    return user


def __create_user_interactive_core(userdata):
    i_user, i_user_created = create_or_update_interactive_user(
        user_id=None, data=userdata, audit_user_id=999, connected=False
    )
    create_or_update_core_user(
        user_uuid=None, username=userdata["username"], i_user=i_user
    )
    return DbManagerUtils.get_object_or_none(User, username=userdata["username"])



from api_fhir_r4.configurations import IssueTypeConfiguration


class R4IssueTypeConfig(IssueTypeConfiguration):

    @classmethod
    def build_configuration(cls, cfg):
        cls.get_config().R4_fhir_issue_type_config = cfg["R4_fhir_issue_type_config"]

    @classmethod
    def get_fhir_code_for_exception(cls):
        return cls.get_config_attribute("R4_fhir_identifier_type_config").get(
            "fhir_code_for_exception", "exception"
        )

    @classmethod
    def get_fhir_code_for_not_found(cls):
        return cls.get_config_attribute("R4_fhir_identifier_type_config").get(
            "fhir_code_for_not_found", "not-found"
        )

    @classmethod
    def get_fhir_code_for_informational(cls):
        return cls.get_config_attribute("R4_fhir_identifier_type_config").get(
            "fhir_code_for_informational", "informational"
        )



from claim.models import Claim

from api_fhir_r4.configurations import R4CommunicationRequestConfig as Config
from api_fhir_r4.tests import GenericTestMixin
from claim.test_helpers import create_test_claim_context


class CommunicationRequestTestMixin(GenericTestMixin):
    _TEST_CODE = "codeTest"
    _TEST_UUID = "7ac646cb-d3cd-4660-baeb-ee34ecf0354e"
    _TEST_STATUS = Claim.STATUS_ENTERED
    _TEST_STATUS_DISPLAY = "entered"
    _TEST_OUTCOME = "queued"
    _TEST_ADJUSTMENT = "adjustment"
    _TEST_DATE_PROCESSED = "2010-11-16T00:00:00"
    _TEST_APPROVED = 1000.00
    _TEST_REJECTION_REASON = 0
    _TEST_VISIT_TYPE = "O"

    # claim item data
    _TEST_ITEM_CODE = "iCode"
    _TEST_ITEM_UUID = "e2bc1546-390b-4d41-8571-632ecf7a0936"
    _TEST_ITEM_STATUS = Claim.STATUS_ENTERED
    _TEST_ITEM_QUANTITY = 20
    _TEST_ITEM_PRICE = 10.0
    _TEST_ITEM_REJECTED_REASON = 0

    # claim service data
    _TEST_SERVICE_CODE = "sCode"
    _TEST_SERVICE_UUID = "a17602f4-e9ff-4f42-a6a4-ccefcb23b4d6"
    _TEST_SERVICE_STATUS = Claim.STATUS_ENTERED
    _TEST_SERVICE_QUANTITY = 1
    _TEST_SERVICE_PRICE = 800
    _TEST_SERVICE_REJECTED_REASON = 0

    _TEST_ID = 9999
    _PRICE_ASKED = 1000
    _PRICE_APPROVED = 1000
    _ADMIN_AUDIT_USER_ID = 1

    _TEST_ITEM_AVAILABILITY = True

    _TEST_ITEM_TYPE = "D"
    _TEST_SERVICE_TYPE = "D"

    # insuree and claim admin data
    _TEST_INSUREE_UUID = "76aca309-f8cf-4890-8f2e-b416d78de00b"
    _TEST_CLAIM_ADMIN_UUID = "044c33d1-dbf3-4d6a-9924-3797b461e535"

    # hf test data
    _TEST_HF_ID = 10000
    _TEST_HF_UUID = "6d0eea8c-62eb-11ea-94d6-c36229a16c2f"
    _TEST_HF_CODE = "12345678"
    _TEST_HF_NAME = "TEST_NAME"
    _TEST_HF_LEVEL = "H"
    _TEST_HF_LEGAL_FORM = "G"
    _TEST_ADDRESS = "TEST_ADDRESS"
    _TEST_PHONE = "133-996-476"
    _TEST_FAX = "1-408-999 8888"
    _TEST_EMAIL = "TEST@TEST.com"
    test_claim_admin = None
    test_insuree = None
    test_claim = None
    test_hf = None
    test_village = None
    sub_str = {}

    def setUp(self):
        super(CommunicationRequestTestMixin, self).setUp()
        service_props = {
            "status": self._TEST_SERVICE_STATUS,
            "qty_approved": self._TEST_SERVICE_QUANTITY,
            "qty_provided": self._TEST_SERVICE_QUANTITY,
            "rejection_reason": self._TEST_SERVICE_REJECTED_REASON,
            "availability": self._TEST_ITEM_AVAILABILITY,
            "price_asked": self._TEST_SERVICE_PRICE,
            "price_approved": self._TEST_SERVICE_PRICE,
            "audit_user_id": self._ADMIN_AUDIT_USER_ID,
        }
        item_props = {
            "status": self._TEST_ITEM_STATUS,
            "qty_approved": self._TEST_ITEM_QUANTITY,
            "qty_provided": self._TEST_ITEM_QUANTITY,
            "rejection_reason": self._TEST_ITEM_REJECTED_REASON,
            "availability": self._TEST_ITEM_AVAILABILITY,
            "price_asked": self._TEST_ITEM_PRICE,
            "price_approved": self._TEST_ITEM_PRICE,
            "audit_user_id": self._ADMIN_AUDIT_USER_ID,
        }

        self.test_claim, self.test_insuree, policy, self.test_hf = (
            create_test_claim_context(
                claim={
                    "visit_type": self._TEST_VISIT_TYPE,
                    "status": self._TEST_STATUS,
                    "feedback_status": Claim.FEEDBACK_SELECTED,
                },
                claim_admin={
                    "uuid": self._TEST_CLAIM_ADMIN_UUID,
                },
                insuree={"uuid": self._TEST_INSUREE_UUID},
                product={},
                hf={
                    "uuid": self._TEST_HF_UUID,
                    "name": self._TEST_HF_NAME,
                    "level": self._TEST_HF_LEVEL,
                    "legal_form_id": self._TEST_HF_LEGAL_FORM,
                    "address": self._TEST_ADDRESS,
                    "phone": self._TEST_PHONE,
                    "fax": self._TEST_FAX,
                    "email": self._TEST_EMAIL,
                },
                items=[item_props],
                services=[service_props],
            )
        )

        self.sub_str[self._TEST_HF_UUID] = self.test_hf.uuid
        self.sub_str[self._TEST_CLAIM_ADMIN_UUID] = self.test_claim.admin.uuid
        self.sub_str[self._TEST_INSUREE_UUID] = self.test_insuree.uuid
        self.sub_str[self._TEST_UUID] = self.test_claim.uuid
        self.sub_str[self._TEST_SERVICE_UUID] = (
            self.test_claim.services.first().service.uuid
        )
        self.sub_str[self._TEST_ITEM_UUID] = self.test_claim.items.first().item.uuid

    def create_test_imis_instance(self):

        return self.test_claim

    def verify_fhir_instance(self, fhir_obj):
        self.assertEqual("active", fhir_obj.status)
        self.assertEqual(
            f"{Claim.FEEDBACK_SELECTED}", fhir_obj.statusReason.coding[0].code
        )
        self.assertIn(str(self.test_insuree.uuid), fhir_obj.subject.reference)
        self.assertIn(str(self.test_claim.admin.uuid), fhir_obj.recipient[0].reference)
        self.assertIn(str(self.test_claim.uuid), fhir_obj.about[0].reference)
        for payload in fhir_obj.payload:
            code = payload.extension[0].valueCodeableConcept.coding[0].code
            content_string = payload.contentString
            if code == Config.get_fhir_care_rendered_code():
                self.assertEqual("Care Rendered? (yes|no)", content_string)
            elif code == Config.get_fhir_payment_asked_code():
                self.assertEqual("Payment Asked? (yes|no)", content_string)
            elif code == Config.get_fhir_drug_prescribed_code():
                self.assertEqual("Drug Prescribed? (yes|no)", content_string)
            elif code == Config.get_fhir_drug_received_code():
                self.assertEqual("Drug Received? (yes|no)", content_string)
            elif code == Config.get_fhir_asessment_code():
                self.assertEqual("Asessment? (0|1|2|3|4|5)", content_string)



from api_fhir_r4.converters import CommunicationConverter
from api_fhir_r4.tests import CommunicationTestMixin
from fhir.resources.R4B.communication import Communication

from api_fhir_r4.tests.mixin import (
    ConvertToImisTestMixin,
    ConvertToFhirTestMixin,
    ConvertJsonToFhirTestMixin,
)


class CommunicationConverterTestCase(
    CommunicationTestMixin,
    ConvertToImisTestMixin,
    ConvertToFhirTestMixin,
    ConvertJsonToFhirTestMixin,
):
    converter = CommunicationConverter
    fhir_resource = Communication
    json_repr = "test/test_communication.json"



from api_fhir_r4.configurations import LocationConfiguration


class R4LocationConfig(LocationConfiguration):

    @classmethod
    def build_configuration(cls, cfg):
        cls.get_config().R4_fhir_location_type = cfg["R4_fhir_location_site_type"]
        cls.get_config().R4_fhir_location_physical_type = cfg[
            "R4_fhir_location_physical_type"
        ]
        cls.get_config().R4_fhir_hf_service_type = cfg["R4_fhir_hf_service_type"]
        cls.get_config().R4_fhir_location_status_codes = cfg[
            "R4_fhir_location_status_codes"
        ]
        cls.get_config().R4_fhir_location_subscribe_signal = cfg[
            "R4_fhir_location_subscribe_signal"
        ]

    @classmethod
    def get_fhir_location_site_type_system(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "system", "http://hl7.org/fhir/v3/ServiceDeliveryLocationRoleType/vs.html"
        )

    @classmethod
    def get_fhir_code_for_hospital(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "fhir_code_for_hospital", "HOSP"
        )

    @classmethod
    def get_fhir_code_for_dispensary(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "fhir_code_for_dispensary", "COMM"
        )

    @classmethod
    def get_fhir_code_for_health_center(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "fhir_code_for_health_center", "OF"
        )

    @classmethod
    def get_fhir_location_area_type_system(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "system",
            "https://openimis.atlassian.net/wiki/spaces/OP/pages/1389297887/FHIR+r4+-+Location",
        )

    @classmethod
    def get_fhir_code_for_region(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "fhir_code_for_region", "R"
        )

    @classmethod
    def get_fhir_code_for_district(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "fhir_code_for_district", "D"
        )

    @classmethod
    def get_fhir_code_for_ward(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "fhir_code_for_ward", "W"
        )

    @classmethod
    def get_fhir_code_for_village(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "fhir_code_for_village", "V"
        )

    @classmethod
    def get_fhir_location_physical_type_system(cls):
        return cls.get_config_attribute("R4_fhir_location_physical_type").get(
            "system",
            "https://openimis.github.io/openimis_fhir_r4_ig/CodeSystem/location-type",
        )

    @classmethod
    def get_fhir_code_for_site(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "fhir_code_for_site", "si"
        )

    @classmethod
    def get_fhir_code_for_area(cls):
        return cls.get_config_attribute("R4_fhir_location_type").get(
            "fhir_code_for_area", "area"
        )

    # This should be part organisation because this is not related to the buiilding

    @classmethod
    def get_fhir_hf_service_type_system(cls):
        return cls.get_config_attribute("R4_fhir_hf_service_type").get(
            "system", "http://hl7.org/fhir/valueset-service-type.html"
        )

    @classmethod
    def get_fhir_code_for_in_patient(cls):
        return cls.get_config_attribute("R4_fhir_hf_service_type").get(
            "fhir_code_for_in_patient", "I"
        )

    @classmethod
    def get_fhir_code_for_out_patient(cls):
        return cls.get_config_attribute("R4_fhir_hf_service_type").get(
            "fhir_code_for_out_patient", "O"
        )

    @classmethod
    def get_fhir_code_for_both(cls):
        return cls.get_config_attribute("R4_fhir_hf_service_type").get(
            "fhir_code_for_both", "B"
        )

    @classmethod
    def get_fhir_code_for_active(cls):
        return cls.get_config_attribute("R4_fhir_location_status_codes").get(
            "fhir_code_for_active", "active"
        )

    @classmethod
    def get_fhir_code_for_inactive(cls):
        return cls.get_config_attribute("R4_fhir_location_status_codes").get(
            "fhir_code_for_inactive", "inactive"
        )

    @classmethod
    def get_subscribe_location_signal(cls):
        return cls.get_config_attribute("R4_fhir_location_subscribe_signal")



import datetime

from django.db.models import Prefetch
from rest_framework.viewsets import GenericViewSet
from api_fhir_r4.permissions import FHIRApiCoverageRequestPermissions
from api_fhir_r4.serializers import ContractSerializer
from api_fhir_r4.views.fhir.base import BaseFHIRView
from api_fhir_r4.mixins import RetrieveModelMixin, ListModelMixin
from api_fhir_r4.views.filters import ValidityFromRequestParameterFilter
from insuree.models import InsureePolicy
from policy.models import Policy


class ContractViewSet(BaseFHIRView, RetrieveModelMixin, ListModelMixin, GenericViewSet):
    lookup_field = "uuid"
    serializer_class = ContractSerializer
    permission_classes = (FHIRApiCoverageRequestPermissions,)

    def list(self, request, *args, **kwargs):
        queryset = (
            self.get_queryset()
            .select_related("product")
            .select_related("officer")
            .select_related("family__head_insuree")
            .select_related("family__location")
            .prefetch_related(
                Prefetch(
                    "insuree_policies",
                    queryset=InsureePolicy.objects.filter(
                        validity_to__isnull=True
                    ).select_related("insuree"),
                )
            )
        )
        refDate = request.GET.get("refDate")
        refEndDate = request.GET.get("refEndDate")
        identifier = request.GET.get("identifier")
        if identifier:
            queryset = queryset.filter(chf_id=identifier)
        else:
            queryset = queryset.filter(validity_to__isnull=True).order_by(
                "validity_from"
            )
            if refDate is not None:
                try:
                    datevar = datetime.datetime.strptime(refDate, "%Y-%m-%d").date()
                except ValueError:
                    pass
                finally:
                    queryset = queryset.filter(validity_from__gte=datevar)
            if refEndDate is not None:
                try:
                    datevar = datetime.datetime.strptime(refEndDate, "%Y-%m-%d").date()
                except ValueError:
                    pass
                finally:
                    queryset = queryset.filter(validity_from__lt=datevar)

        serializer = ContractSerializer(
            self.paginate_queryset(queryset), many=True, user=request.user
        )
        return self.get_paginated_response(serializer.data)

    def get_queryset(self):
        queryset = Policy.get_queryset(None, self.request.user)
        return ValidityFromRequestParameterFilter(self.request).filter_queryset(
            queryset
        )



import copy

from location.models import Location

from api_fhir_r4.converters import LocationConverter
from api_fhir_r4.serializers.baseSerializer import BaseFHIRSerializer


class LocationSerializer(BaseFHIRSerializer):
    fhirConverter = LocationConverter

    def create(self, validated_data):
        copied_data = copy.deepcopy(validated_data)
        if "_state" in copied_data:
            del copied_data["_state"]
        return Location.objects.create(**copied_data)

    def update(self, instance, validated_data):
        instance.code = validated_data.get("code", instance.code)
        instance.name = validated_data.get("name", instance.name)
        instance.type = validated_data.get("type", instance.type)
        instance.parent_id = validated_data.get("parent_id", instance.parent_id)
        instance.audit_user_id = self.get_audit_user_id()
        instance.save()
        return instance
